#!/usr/bin/perl 
use SNMP;
use Config::IniFiles;
umask(0022);

$config = Config::IniFiles->new( -file => "config.ini" );

$dir        = &getVal($config, "GLOBAL", "dir");
$configsDir = $dir."/configs";
$dataDir    = $dir."/data";
$utilDir    = $dir."/util";
$layoutDir  = $dir."/layout";
$graphDir   = &getVal($config, "GLOBAL", "graph_dir");
$version    = &getVal($config, "GLOBAL", "version");

opendir(dirHandel, $configsDir) || die "Failed opening configs dir\n";
@configsFile = readdir(dirHandel);
closedir(dirHandel);

use Config::IniFiles;
my $cfg = new Config::IniFiles();
print "The value is " . $cfg->SetFileName() . "."
	if $cfg->val( 'GLOBAL', 'AAA' );


if($ARGV[0] eq "html")
{
	&create_html();
}elsif($ARGV[0] eq "chkdata")
{
	print &getData($ARGV[1]);
}else
{
	&main();
}



sub main
{
#	my($inicfg);
	foreach $file (@configsFile)
	{
		-f $configsDir."/".$file || next;
		undef $inicfg;
		$inicfg = new Config::IniFiles(-file => $configsDir."/".$file);
		&create_check($inicfg);
		&update($inicfg);
		&graph_all($inicfg);
	}
}

sub create_check
{
	my($inicfg)		= $_[0];
	my($hostName)	= &getVal($inicfg, "GLOBAL", "NAME");
	my(@sections)   = $inicfg->Sections();

	foreach $section (@sections)
	{
		$section eq "GLOBAL" && next;
		-e $dataDir."/".$hostName.".".$section.".rrd" || &create($inicfg, $section);
	}
}

sub create
{
	my($inicfg)		= $_[0];
	my($section)    = $_[1];
	my($cmd)  = &getVal($config, "GLOBAL", "rrdcmd")." create ";

	$cmd .= $dataDir."/".&getVal($inicfg, "GLOBAL", "NAME").".".$section.".rrd ";
	$cmd .= "--step 300 ";
	for($i=1;$i<20;$i++)
	{
		if($gCfg = &getVal($inicfg, $section,"DATA".$i,""))
		{
			my(@tmpExpCfg) = split(":", $gCfg);
			$cmd    .= "DS:D".$i.":".$tmpExpCfg[1].":600:"
			          .$tmpExpCfg[2].":"
			          .$tmpExpCfg[3]." ";
		}
	}
    $cmd .= "RRA:AVERAGE:0.5:1:600 "
           ."RRA:AVERAGE:0.5:6:700 "
           ."RRA:AVERAGE:0.5:24:775 "
           ."RRA:AVERAGE:0.5:288:797 "
           ."RRA:MAX:0.5:1:600 "
           ."RRA:MAX:0.5:6:700 "
           ."RRA:MAX:0.5:24:775 "
           ."RRA:MAX:0.5:288:797 ";
#	print $cmd."\n\n";
	print `$cmd`;
}

sub update
{
	my($inicfg)		= $_[0];

	my($hostName)	= &getVal($inicfg, "GLOBAL", "NAME");
	my(@sections)    = $inicfg->Sections();
	my($cmd);
	my($fFlag);
	my($tCmd);

	foreach $section (@sections)
	{
		$section eq "GLOBAL" && next;

		$cmd = &getVal($config, "GLOBAL", "rrdcmd")
			  ." update ".$dataDir."/"
		      .&getVal($inicfg, "GLOBAL", "NAME")."."
			  .$section.".rrd --template ";

		for($i=1,$fFlag=0,$tCmd=$gData="";$i<20;$i++)
		{
			if($gCfg = &getVal($inicfg, $section,"DATA".$i,""))
			{
				if($fFlag){$tCmd .= ":";}
				$tCmd .= "D".$i;
				$fFlag++;
				$gData .= ":".&getData($gCfg);
			}
		}
		$cmd .= $tCmd." N".$gData;
#		print $cmd."\n";
		print `$cmd`;
	}
}

sub getData
{
	my($cmd);
	my($snmp);
	my($retVar);
	my(@sarg)  = split(/:/, $_[0]);
	#print "$sarg[4] $sarg[5] $sarg[6]\n";
	if($sarg[0] =~ /SNMP/)
	{
		$snmp = new SNMP::Session(DestHost => $sarg[4], Community => $sarg[5], TIMEOUT => 3000000);
		$retVar = $snmp->get($sarg[6]);
		unless($sarg[7] || $sarg[8])
		{
			return $retVar;
		}else
		{
			unless($sarg[8]){ return substr($retVar, $sarg[7]);}
			else         { return substr($retVar, $sarg[7], $sarg[8]);}
		}
	}elsif($sarg[0] =~ /CMD/)
	{
        shift(@sarg);
        shift(@sarg);
        shift(@sarg);
        shift(@sarg);
		$cmd = join(':', @sarg);
		$cmd =~ s/__UTIL__/$utilDir/g;
		$cmd =~ s/__DIR__/$dir/g;
		$retVal = `$cmd`;
		$retVal =~ s/^[ \n\t]+|\n$//g;
		return $retVal;
	}
}

sub graph_all
{
	my($inicfg)		= $_[0];
	my(@sections)    = $inicfg->Sections();
	my($cmd);
	my($fFlag);
	
	foreach $section (@sections)
	{
		$section eq "GLOBAL" && next;

		&graph($inicfg, $section, 'h');
		&graph($inicfg, $section, 'd');
		&graph($inicfg, $section, 'w');
		&graph($inicfg, $section, 'm');
		&graph($inicfg, $section, 'y');
	}
}

sub graph
{
	my($inicfg)		 = $_[0];
	my($section)     = $_[1];
	my($type)        = $_[2];
	my($imageOnly)   = $_[3] eq '1'?1:0;
	my($lineGp)      = 1;
	my($hostName)	= &getVal($inicfg, "GLOBAL", "NAME");
	my($extName)     = &getVal($inicfg, "GLOBAL", "IMGTYPE","GIF");
	my($cmd)			= &getVal($config,"GLOBAL", "rrdcmd")." graph ";

	-e $graphDir || mkdir($graphDir, 0777);
	-e $graphDir."/images" || mkdir($graphDir."/images", 0777);

	if($type eq "h")
	{
		$cmd .= $graphDir."/images/".$hostName.".".$section.".hourly.".$extName;
		$cmd .= " --start -18000 ";
	}elsif($type eq "d")
	{
		$cmd .= $graphDir."/images/".$hostName.".".$section.".daily.".$extName;
		$cmd .= " --start -86400 ";
	}elsif($type eq "w")
	{
		$cmd .= $graphDir."/images/".$hostName.".".$section.".weekly.".$extName;
		$cmd .= " --start -604800 ";
	}elsif($type eq "m")
	{
		$cmd .= $graphDir."/images/".$hostName.".".$section.".monthly.".$extName;
		$cmd .= " --start -2678400 ";
	}elsif($type eq "y")
	{
		$cmd .= $graphDir."/images/".$hostName.".".$section.".yearly.".$extName;
		$cmd .= " --start -33053184 ";
	}

	$cmd .= '--imgformat="'.$extName."\" ";
	$cmd .= '--width="'.&getVal($inicfg, $section, "WIDTH", "400").'" ';
	$cmd .= '--height="'.&getVal($inicfg, $section, "height", "100").'" ';
	$cmd .= &getVal($inicfg, $section, "HIDSGRID")?'--no-minor ':'';
	$cmd .= &getVal($inicfg, $section, "BASE")?'--base="'.&getVal($inicfg, $section, "BASE").'" ':'';
	$cmd .= &getVal($inicfg, $section, "TITLE")?'--title="'.&getVal($inicfg, $section, "TITLE").'" ':'';
	$cmd .= &getVal($inicfg, $section, "YLABLE")?'--vertical-label="'.&getVal($inicfg, $section, "YLABLE").'" ':'';
	$cmd .= &getVal($inicfg, $section, "BASE")?'--base="'.&getVal($inicfg, $section, "BASE").'" ':' ';
	$cmd .= &getVal($inicfg, $section, "GRAPHMAX")?'--upper-limit="'.&getVal($inicfg, $section, "GRAPHMAX").'" ':'';
	$cmd .= &getVal($inicfg, $section, "GRAPHMIN")?'--lower-limit="'.&getVal($inicfg, $section, "GRAPHMIN").'" ':'';
	$cmd .= &getVal($config, "THEME", &getVal($inicfg, $section, "GRAPHTHEME","WHITE"), "").' ';

	for($i=1;$i<20;$i++)
	{
		if($gCfg = &getVal($inicfg, $section, "DATA$i"))
		{
			$cmd   .= "DEF:DATA$i=\"".$dataDir."/".$hostName.".".$section.".rrd\":"
				     ."D$i:AVERAGE ";
		}
	}

	for($i=1;$i<20;$i++)
	{
		if($gCfg = &getVal($inicfg, $section, "CDATA$i"))
		{
			@expCfg = split(/:/,$gCfg);
			$cmd   .= "CDEF:".$expCfg[0]."=";
			shift(@expCfg);
			$cmd   .= join(',', @expCfg).' ';
		}
	}

	for($i=1;$i<14;$i++)
	{
		if($gCfg = &getVal($inicfg, $section, "GRAPH$i"))
		{
			@expCfg = split(/:/,$gCfg);
			if($expCfg[2] eq "LINE")
			{
				$expCfg[2] .= $lineGp;
				$lineGp++;
			}

			$cmd .= $expCfg[2].":".$expCfg[0].$expCfg[3];
			if($imageOnly != 1 && $expCfg[4] !~ /NP/)
			{
				$cmd .= ":\"".$expCfg[1]."\" "
				       ."GPRINT:".$expCfg[0].':LAST:"Current\:%8.2lf %s" '
					   ."GPRINT:".$expCfg[0].':AVERAGE:"Average\:%8.2lf %s" '
					   ."GPRINT:".$expCfg[0].':MAX:"Maximum\:%8.2lf %s\\n" ';
			}else
			{
				$cmd .= " ";
			}
		}
	}
	$cmd .= "COMMENT:\"(C)2003. Powered By RRDBELT.(http://www.jinoos.com/rrdbelt)\" ";
#	print $cmd."\n\n";
	`$cmd`;
	
}

sub create_html
{
	my(@host);
	my(@layout);
	my(@imgExt);
	my($hostlist);
	my($hostname);
	@sections;
	foreach $file (@configsFile)
	{
		-f $configsDir."/".$file || next;
		$inicfg					= Config::IniFiles->new( -file => $configsDir."/".$file);

		$hostname               = &getVal($inicfg, "GLOBAL", "NAME");
		@sections               = $inicfg->Sections();
		&create_html_summary($hostname
			                ,*sections
			                ,&getVal($inicfg, "GLOBAL", "LAYOUT", "default")
			                ,&getVal($inicfg, "GLOBAL", "IMGTYPE"));
		$hostlist .= "<b><li><a href='".$hostname.".index.html'>".$hostname."</a></li></b>\n";
	}

	open(fp, $layoutDir."/index.html")||die("Layout file not found : ".$layoutDir."/index.html");
	$str = join(/\n/,<fp>);
	close(fp);

	$str =~ s/__VERSION__/$version/g;
	$str =~ s/__HOSTLIST__/$hostlist/g;

	-e $graphDir || mkdir($graphDir, 0777);

	open(fp, ">".$graphDir."/index.html")||die("Layout file not found : ".$graphDir."/index.html");
	print fp $str;
	close(fp);
	print "Create File ".$graphDir."/index.html\n";
}

sub create_html_summary
{
	my($hostname)         = $_[0];
	*sections             = $_[1];
	my($layout)           = $_[2];
	my($imgExt)           = $_[3];

	my($section);
	my($str);
	my($blockStr);
	my($nStr);
	my($nnStr)            = "";

	open(fp, $layoutDir."/".$layout.".sum.html")||die("Layout file not found : ".$layoutDir."/".$layout.".sum.html");
	$str = join(/\n/,<fp>);
	close(fp);
	$blockStr = &block_replace(\$str, "LOOP");
	foreach $section (@sections)
	{
		$section =~ /GLOBAL/ && next; 
		&create_html_main($hostname, $section, $layout, $imgExt);
		$nStr   = $blockStr;
		$nStr   =~ s/\{TARGET\}/$hostname.$section.html/;
		$nStr   =~ s/\{IMGSRC\}/images\/$hostname.$section.daily.$imgExt/;
		$nnStr .= $nStr;
	}

	$str =~ s/__LOOP__/$nnStr/;
	$str =~ s/__VERSION__/$version/g;

	open(fp, ">".$graphDir."/".$hostname.".index.html")||die("Layout file not found : ".$graphDir."/".$hostname.".index.html");
	print fp $str;
	close(fp);
	print "Create File ".$graphDir."/".$hostname.".index.html\n";
}

sub create_html_main
{
	my($hostname)         = $_[0];
	my($section)          = $_[1];
	my($layout)           = $_[2];
	my($imgExt)           = $_[3];

	my($str);

	open(fp, $layoutDir."/".$layout.".main.html")||die("Layout file not found : ".$layoutDir."/".$layout.".main.html");
	$str = join(/\n/,<fp>);
	close(fp);

	$str =~ s/\{HOURSRC\}/images\/$hostname.$section.hourly.$imgExt/g;
	$str =~ s/\{DAILYSRC\}/images\/$hostname.$section.daily.$imgExt/g;
	$str =~ s/\{WEEKLYSRC\}/images\/$hostname.$section.weekly.$imgExt/g;
	$str =~ s/\{MONTHLYSRC\}/images\/$hostname.$section.monthly.$imgExt/g;
	$str =~ s/\{YEARLYSRC\}/images\/$hostname.$section.yearly.$imgExt/g;
	$str =~ s/__VERSION__/$version/g;

	open(fp, ">".$graphDir."/".$hostname.".".$section.".html")||die("Layout file not found : ".$graphDir."/".$hostname.".".$section.".html");
	print fp $str;
	close(fp);
	print "Create File ".$graphDir."/".$hostname.".".$section.".html\n";
}

sub block_replace
{
	$str       = $_[0];
	my($sp)    = index($$str, "{".$_[1]."}");
	my($ep)    = index($$str, '{/'.$_[1].'}');
	if($sp<0 || $ep<0){return;}
	my($nStr)  = substr($$str, 0, $sp);
	$nStr     .= '__'.$_[1].'__';
	$nStr     .= substr($$str, ($ep+length($_[1])+3));
	$ret       = substr($$str, ($sp+length($_[1])+2), ($ep-($sp+length($_[1])+2)));
	$$str       = $nStr;
	return $ret;
}

sub getVal
{
	my($retVal);
	if($_[3])
	{
		$retVal = $_[0]->val($_[1],$_[2],$_[3])
	}else
	{
		$retVal = $_[0]->val($_[1],$_[2]);
	}
	$retVal =~ s/^\"|\"$//g;
	return $retVal;
}